// C/C++ headers

#include <stdio.h>
#include <stdlib.h>

#include <iostream>
using std::endl;
using std::cout;

// QALE headers

#include "PointerSet.h"
#include "cpp_assert.h"

/*****************************************************************************
 * private struct: PointerSet::Entry
 *****************************************************************************/

struct PointerSet::Entry
{
   struct PointerSet::Entry *next;
   struct PointerSet::Entry *prev;
   void *data;
};

/*****************************************************************************
 * Definition of class: PointerSet
 *****************************************************************************/

PointerSet::PointerSet(DeleteFunction deleteFunction)
   :
   mEntries(NULL),
   mDeleteFunction(deleteFunction)
{
   mEntries = new Entry *[SIZE];

   for(int i = 0; i < SIZE; i ++)
      mEntries[i] = NULL;
}

PointerSet::~PointerSet()
{
   for(int i = 0; i < SIZE; i ++)
   {
      if(mEntries[i])
      {
         Entry *ptr = mEntries[i];

         while(ptr)
         {
            Entry *temp = ptr;
            ptr = ptr->next;

            mDeleteFunction(temp->data);

            delete temp;
         }

         mEntries[i] = NULL;
      }
   }

   delete [] mEntries;
}

void PointerSet::add(void *thePointer)
{
   int result = PointerSet::hashCode(thePointer);

   Entry *thisEntry = new Entry();
   thisEntry->next  = NULL;
   thisEntry->prev  = NULL;
   thisEntry->data  = thePointer;

   if(mEntries[result])
   {
      thisEntry->next        = mEntries[result];
      mEntries[result]->prev = thisEntry;
   }

   mEntries[result] = thisEntry;
}

void *PointerSet::remove(void *thePointer)
{
   int result = PointerSet::hashCode(thePointer);

   for(Entry *index = mEntries[result]; index; index = index->next)
   {
      if(index->data == thePointer)
      {
         if(index == mEntries[result])
         {
            // We're at the beginning of the list

            if(index->next)
               index->next->prev = NULL;

            mEntries[result]  = index->next;
         }

         else if(index->next == NULL)
         {
            // This is the last element in the list (and we also
            // know it isn't the first element in the list

            index->prev->next = NULL;
         }

         else
         {
            // Neither the first nor last node in the list

            index->prev->next = index->next;
            index->next->prev = index->prev;
         }

         delete index;
         return(thePointer);
      }
   }

   // Not found

   return(0);
}

int PointerSet::main(int, char **)
{
   PointerSet p1(PointerSet::genericDelete);

   int **array1 = new int *[1000];
   int i        = 0;

   for(i = 0; i < 1000; i ++)
   {
      array1[i] = new int;
      p1.add(array1[i]);
   }

   p1.print();

   for(i = 0; i < 1000; i ++)
      p1.remove(array1[i]);

   p1.print();

   for(i = 0; i < 1000; i ++)
      p1.add(array1[i]);

   delete [] array1;

   PointerSet p2(PointerSet::genericArrayDelete);
   char **array2 = new char *[1000];

   for(i = 0; i < 1000; i ++)
   {
      array2[i] = new char[10];
      p2.add(array2[i]);
   }

   p2.print();

   for(i = 0; i < 1000; i ++)
      p2.remove(array2[i]);

   p2.print();

   for(i = 0; i < 1000; i ++)
      p2.add(array2[i]);

   delete [] array2;

   return(0);
}

void PointerSet::print() const
{
   for(int i = 0; i < SIZE; i ++)
   {
      if(mEntries[i])
      {
         cout << "Entries[" << i << "] {" << endl;

         for(Entry *index = mEntries[i]; index; index = index->next)
            cout << "   " << index->data << endl;

         cout << "}" << endl;
      }
   }
}

void PointerSet::genericDelete(void *ptr)
{
   // g++ warns that 'void *' is not a pointer to object type
   // without this cast

   delete (char *)ptr;
}

void PointerSet::genericArrayDelete(void *ptr)
{
   // g++ warns that 'void *' is not a pointer to object type
   // without this cast

   delete [] (char *)ptr;
}

int PointerSet::hashCode(void *thePointer)
{
   static int checkedAssertion = 0;

   if(!checkedAssertion)
   {
      CPP_ASSERT(sizeof(void *) == sizeof(long));
      checkedAssertion = 1;
   }

   long value        = (long)thePointer;
   unsigned char *uc = (unsigned char *)&value;
   int result        = uc[0];

   for(int i = 1; i < (int)sizeof(long); i ++)
      result ^= uc[i];

   return(result);
}

#ifdef __MAIN__

int main(int argc, char **argv)
{
   return(PointerSet::main(argc, argv));

}  /* main */

#endif
