// C/C++ headers

#include <stdio.h>
#include <string>

using std::string;

// QALE headers

#include "PropertyVisitor.h"
#include "PropertyNode.h"
#include "Widget.h"
#include "MessageReceiver.h"
#include "Component.h"

/*****************************************************************************
 * Definition of class: PropertyVisitor
 *****************************************************************************/

PropertyVisitor::PropertyVisitor(MessageReceiver &receiver)
   : mReceiver(receiver)
{
   // Nothing to do
}

PropertyVisitor::~PropertyVisitor()
{
   // Nothing to do
}

void PropertyVisitor::visited(AttributeNode *) {}
void PropertyVisitor::visited(BinaryNode *)    {}
void PropertyVisitor::visited(ConstantNode *)  {}
void PropertyVisitor::visited(DefineNode *)    {}

void PropertyVisitor::visited(PropertyNode *propertyNode)
{
   char *qualifiedName     = propertyNode->getQualifiedName();
   char *propertyName      = propertyNode->getPropertyName();
   Component *theComponent = NULL;

   if(*qualifiedName)
   {
      Widget *theWidget = findWidget(qualifiedName);

      if(theWidget && theWidget->getComponent()->hasProperty(propertyName))
         theComponent = theWidget->getComponent();
   }
   else
   {
      // Unlike defines we begin searching from our current widget
      // rather than from our parent (this is to allow our top-level
      // widget to use properties)

      Widget *theWidget = getCurrentWidget();

      while(theWidget)
      {
         if(theWidget->getComponent()->hasProperty(propertyName))
         {
            theComponent = theWidget->getComponent();
            break;
         }

         theWidget = theWidget->getParent();
      }
   }

   if(!theComponent)
   {
      string errMsg = getErrorMessage("undefined property", propertyNode);
      mReceiver.accept(errMsg.c_str());
   }
   else
   {
      propertyNode->setComponent(theComponent);
   }
}

void PropertyVisitor::visited(MinMaxNode *)    {}
void PropertyVisitor::visited(UnaryNode *)     {}
