// QALE headers

#include "QMainComponent.h"

// Qt headers

#include <qwidget.h>
#include <qrect.h>

/*****************************************************************************
 * Definition of class: QMainComponent
 *
 * This class sets its size/location the first time setSize/setLocation
 * is called but after these values are set initially it uses its
 * underlying Component object to report back its size/location.  This
 * object represents the top-level widget and we want its size and
 * location to be able to be changed by the user which is why we
 * ignore setSize/setLocation after the initial calls to those
 * methods.
 *****************************************************************************/

QMainComponent::QMainComponent(int index, QWidget *q)
   : QComponent(index, q),
     mSetSizeCount(0),
     mSetLocationCount(0)
{
   // Nothing to do
}

QMainComponent::~QMainComponent()
{
   // Nothing to do
}

void QMainComponent::setLocation(double x, double y)
{
   if(mSetSizeCount++ == 0)
   {
      getWidget()->move(
         (int)(x + 0.5),
         (int)(y + 0.5)
      );
   }
}

void QMainComponent::setSize(double width, double height)
{
   if(mSetLocationCount++ == 0)
   {
      getWidget()->resize(
         (int)(width + 0.5),
         (int)(height + 0.5)
      );
   }
}

void QMainComponent::getLocation(double *xPtr, double *yPtr) const
{
   QRect rect = getWidget()->rect();

   *xPtr = (double)rect.left();
   *yPtr = (double)rect.top();
}

void QMainComponent::getSize(double *widthPtr, double *heightPtr) const
{
   QRect rect = getWidget()->rect();

   *widthPtr  = (double)rect.width();
   *heightPtr = (double)rect.height();
}
